/////////////////////////////////////////////////////////////////////
// Design unit: cordic
//            :
// File name  : cordic_top.sv
//            :
//            :
//
// Author     : Ashley Robinson
//            :ajr2g10@ecs.soton.ac.uk
/////////////////////////////////////////////////////////////////////

module cordic_top (
   output logic data_out, 
   output logic [15:0] X, 
   output logic [15:0] Y, 
   input logic [15:0] x,
   input logic [15:0] y,
   input logic [15:0] a,
   input logic [15:0] b,
   input logic Clk, Reset, Start
);
logic[15:0]
   input_delay_x[16:0],
   input_delay_y[16:0],
   delay_x_buf[16:0],
   delay_y_buf[16:0],
   theta_pass;

logic 
   good_data_pass;

//Delay input data by 16 clk cycles
generate
   genvar j;
   for (j=0; j < 16; j++) 
      begin : delay_stages
         always_ff @(posedge Clk)
         begin
            input_delay_x[j+1] <= input_delay_x[j];
            input_delay_y[j+1] <= input_delay_y[j];
         end
      end
endgenerate

//All combo in the top level
always_comb
begin
   //begin the delay train
   input_delay_x[0] = x;
   input_delay_y[0] = y;
end

//Data first goes through this module
vector_cordic Vec(
   .data_out_vec(good_data_pass), 
   .rootxy(), 
   .atanba(theta_pass), 
   .x(a),
   .y(b),
   .z(16'h0000),
   .Clk(Clk), 
   .Reset(Reset), 
   .Start(Start)
);

//Data out for the vectoring goes here along with data
//delayed from the initial input
rotation_cordic Rot(
   .data_out_rot(data_out), 
   .xprime(X), 
   .yprime(Y), 
   .x(input_delay_x[16]),
   .y(input_delay_y[16]),
   .theta(theta_pass),
   .Clk(Clk), 
   .Reset(Reset), 
   .Start(good_data_pass)
);

endmodule


