/////////////////////////////////////////////////////////////////////
// Design unit: vector_cordic
//            :
// File name  : vector_cordic.sv
//            :
//            :
//
// Author     :Ashley Robinson 
//            :ajr2g10@ecs.soton.ac.uk
/////////////////////////////////////////////////////////////////////


//Generic module for each stage in vector the pipeline
module vector_cordic_stage(
   //data going to next stage 
   output logic[31:0] x_new,
   output logic[31:0] y_new,
   output logic[31:0] z_new,
   output logic[31:0] z_first_new,

   //data from last stage
   input logic signed[31:0] x_old,
   input logic signed[31:0] y_old,
   input logic signed[31:0] z_old,
   input logic signed[31:0] z_first_old,

   //Overheads
   input logic signed [31:0] alpha,
   int i,

   //Pipe controls
   input logic Data_in_ready,
   output logic Data_out_ready,

   //Standard
   input logic Clk,
   input logic Reset
);

//Wires between cloud and reg
logic signed [31:0]
   x_buf,
   y_buf,
   z_buf,
   x_shift,
   y_shift;

//Register after the cloud
always_ff @(posedge Clk)
begin
   if(Reset)
      begin
         x_new <= 32'h00000000;
         y_new <= 32'h00000000;
         z_new <= 32'h00000000;
         z_first_new <= 32'h00000000;
         Data_out_ready <= 1'b0;
      end
   else
      begin
         Data_out_ready <= Data_in_ready;
         z_first_new <= z_first_old;
         if(Data_in_ready)
         begin
            x_new <= x_buf;
            y_new <= y_buf;
            z_new <= z_buf;
         end
      end 
end

//Combo cloud
always_comb
begin
   //Always need to shift
   x_shift = (x_old >>> i);
   y_shift = (y_old >>> i);  

   //sigma = 1 then sign(xy) is +ve
   if(x_old[31] == y_old[31])
      begin
         //sigma = 1
         x_buf = x_old + y_shift;
         y_buf = y_old - x_shift;
         z_buf = z_old - alpha; 
      end
   else
      begin
         //sigma = -1
         x_buf = x_old - y_shift;
         y_buf = y_old + x_shift;
         z_buf = z_old + alpha; 
      end
end
endmodule

