/////////////////////////////////////////////////////////////////////
// Design unit: rotation_cordic_stage
//            :
// File name  : rotation_cordic_stage.sv
//            :
//            :
//
// Author     : Ashley J. Robinson
//            : ajr2g10@ecs.soton.ac.uk
/////////////////////////////////////////////////////////////////////

//Generic module for each stage in the pipeline
module rotation_cordic_stage(
   //data going to next stage 
   output logic[31:0] x_new,
   output logic[31:0] y_new,
   output logic[31:0] theta_new,

   //data from last stage
   input logic signed[31:0] x_old,
   input logic signed[31:0] y_old,
   input logic signed[31:0] theta_old,

   //Overheads
   input logic[31:0] alpha,
   int i,

   //Pipe controls
   input logic Data_in_ready,
   output logic Data_out_ready,

   //Standard
   input logic Clk,
   input logic Reset
);

//Wires between cloud and reg
logic [31:0]
   x_buf,
   y_buf,
   theta_buf,
   x_shift, 
   y_shift;


//Register after the cloud
always_ff @(posedge Clk)
begin
   if(Reset)
      begin
         x_new <= 32'h00000000;
         y_new <= 32'h00000000;
         theta_new <= 32'h00000000;
         Data_out_ready <= 1'b0;
      end
   else
      begin
         Data_out_ready <= Data_in_ready;
         if(Data_in_ready)
         begin
            x_new <= x_buf;
            y_new <= y_buf;
            theta_new <= theta_buf;
         end
      end
end

//Combo cloud
always_comb
begin
   x_shift = (x_old >>> i);
   y_shift = (y_old >>> i);

   //If theta is negative then add to it
   // if not negative then subtract
   //get it closer to zero
   if(theta_old[31])
      begin
         x_buf = x_old + y_shift;
         y_buf = y_old - x_shift;
         theta_buf = theta_old + alpha;
      end
   else
      begin
         x_buf = x_old - y_shift;
         y_buf = y_old + x_shift;
         theta_buf = theta_old - alpha;
      end
end



endmodule
